/*********************************************************************************
 * Includes
 ********************************************************************************/
// Basically, first, as on top of all HAM files, the most
// important include is the mygba.h, which contains all the
// macro definitions, and the HAMlib function prototypes.
#include <mygba.h>
// Include the HEL Library Header
// You can download HEL at http://www.console-dev.de
#include <hel.h>

/*********************************************************************************
 * Prototypes
 ********************************************************************************/
void SomeInterruptFunc(void);
void DumbInterruptType(void *InterruptProc);

/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    // Initialize HAMlib
    // Then we call the single most important function of ham, ham_Init .
    // This function must be called first thing when working with HAMlib,
    // because all ham_* functions will only work correctly after this has been called.
    // Behaviour of HAMlib is undefined if you comment this out.
    ham_Init();

    ham_SetBgMode(0);
    ham_InitText(0);

    // Start an interrupt handler. In this case we start the VBL
    ham_StartIntHandler(INT_TYPE_VBL, (void*)SomeInterruptFunc);
    
    // Lookup what interrupt this function is assigned to!
    DumbInterruptType(&SomeInterruptFunc);
    
    // Loop
    while(TRUE) { }

    return 0;
}


void SomeInterruptFunc(void)
{
    // muhh
}


void DumbInterruptType(void *InterruptProc)
{
   switch(hel_IntrGetType(InterruptProc))
   {
      case INT_TYPE_VBL:
         ham_DrawText(1,1,"INT_TYPE_VBL");
         break;

      case INT_TYPE_HBL:
         ham_DrawText(1,1,"INT_TYPE_HBL");
         break;

      case INT_TYPE_VCNT:
         ham_DrawText(1,1,"INT_TYPE_VCNT");
         break;

      case INT_TYPE_TIM0:
         ham_DrawText(1,1,"INT_TYPE_TIM0");
         break;

      case INT_TYPE_TIM1:
         ham_DrawText(1,1,"INT_TYPE_TIM1");
         break;

      case INT_TYPE_TIM2:
         ham_DrawText(1,1,"INT_TYPE_TIM2");
         break;

      case INT_TYPE_TIM3:
         ham_DrawText(1,1,"INT_TYPE_TIM3");
         break;

      case INT_TYPE_SIO:
         ham_DrawText(1,1,"INT_TYPE_SIO");
         break;

      case INT_TYPE_DMA0:
         ham_DrawText(1,1,"INT_TYPE_DMA0");
         break;

      case INT_TYPE_DMA1:
         ham_DrawText(1,1,"INT_TYPE_DMA1");
         break;

      case INT_TYPE_DMA2:
         ham_DrawText(1,1,"INT_TYPE_DMA2");
         break;

      case INT_TYPE_DMA3:
         ham_DrawText(1,1,"INT_TYPE_DMA3");
         break;

      case INT_TYPE_KEY:
         ham_DrawText(1,1,"INT_TYPE_KEY");
         break;

      case INT_TYPE_CART:
         ham_DrawText(1,1,"INT_TYPE_CART");
         break;

      case 0xff:
         ham_DrawText(1,1,"No interrupt function", (u32)InterruptProc);
         ham_DrawText(1,2,"assigned to 0x%.8x", (u32)InterruptProc);
         break;
   }
}


/* END OF FILE */
