/*
 * This document has been created by using VisualHAM
 * http://www.console-dev.de
 *
 * Author: Peter Schraut
 * Date..: 19th Jan. 2004
 *
 * Example shows how to use HEL's cinema-scrolling-map-system
 *
*/
#include <mygba.h>

// include hel header
#include <hel.h>

//
// defines
//
#define SCROLL_SPEED    1
// comment out for non-callbacks
#define USE_MAP_CALLBACKS

//
// Global variables
//
u8  g_NewFrame=0;
u8  g_Frames=0;
TMapScrollInfo g_MapInfo;
TPoint32 g_CurPos;


//
// X/Y Tile-Position (Key-Points)
//
const TMapCamKeyPoint CamKeyPoints[10]=
{
   {10, 5},
   {10,25},
   {15, 5},
   {15,25},
   { 0, 0},
   {40,10},
   {40,40},
   {20,40},
   {20,20},
   {0,  0}
};

//
// Map Cinema Scrolling Information
//
TMapCamScrollInfo g_MapCamInfo =
{
   (TMapCamKeyPoint*)&CamKeyPoints, // Pointer to keypoints
   10,             // Count of keypoints
   0               // Start keypoint
};


//
// extern graphics
//
extern const unsigned short isle_Map[16384];
extern const unsigned char isle_Tiles[9088];
extern const unsigned short master_Palette[256];


//
// Prototypes
//
void vblFunc(void);
void MapScroll(void);
void MapOnTopRowChanged(TMapScrollInfo *pMapInfo, u32 X, u32 Y);
void MapOnLeftColumnChanged(TMapScrollInfo *pMapInfo, u32 X, u32 Y);


/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    // Initialize HAMlib
    // Then we call the single most important function of ham, ham_Init .
    // This function must be called first thing when working with HAMlib,
    // because all ham_* functions will only work correctly after this has been called.
    // Behaviour of HAMlib is undefined if you comment this out.
    ham_Init();
    
    // Sets the GBAs BGMode to the value specified in bgno
    ham_SetBgMode(0);
    
    // Init the Text display system on the bg of your choice
    // This function must be called if you want to use HAMlibs text display system.
    // After calling this, you can use ham_DrawText and ham_SetTextCol.
    ham_InitText(0);
    
    // Load Background Palette
    ham_LoadBGPal256((void *)&master_Palette);
    
    // Will set the color indices of the font to the BG colors specified
    ham_SetTextCol(29, 0);
    
    // This function will load a tileset (usually from ROM)
    // from the src pointer and copy it into an appropiate spot
    // in the GBAs BG VRAM. The spot is decided by the memory manager,
    // and passed back in the structure.
    ham_bg[2].ti = ham_InitTileSet((void*)&isle_Tiles,              // A pointer to the source data
                                    SIZEOF_16BIT(isle_Tiles),       // The size of the tiles to be copied into Tile RAM, in number of 16bit chunks
                                    1,                              // The Color mode of the tile set (0=16col 1=256col)
                                    1);                             // Setting this one requires a bit of understanding of the GBA hardware.
                                                                    // Basically, if you put tiles into the GBA memory, you will have to put
                                                                    // them at the start of a Character Base block in order to be able to
                                                                    // access tile #0 with a map entry of 0. If the allocated block is not
                                                                    // directly at the start of a CBB, you will have to offset your map entries.
                                                                    // This is automatically done for you when you use ham_SetMapTile.
                                                                    // However, when you load your map from ROM, you might want to align your
                                                                    // tileset to a CBB boundary. If in doubt, and you have some VRAM left, set
                                                                    // this to 1 always.
                                                                    // 0=allocate at the earliest possible spot, unsafe if you do not know / do not
                                                                    // want to offset your map entries!
                                                                    // 1=allocate only on a CBB start to ensure tile 0 is map index no 0.
                                                                    // Again, if you have no idea what this is, set it to 1.
                        
    // Init map-system for bg 2   
    g_MapInfo = hel_MapInit(2,        // background
                          1024/8,   // mapwidth in tiles
                          1024/8,   // mapheight in tiles
                          FALSE,    // rotation map?
                          (void*)isle_Map);   // pointer to mapdata

#ifdef USE_MAP_CALLBACKS
    hel_MapSetCallbacks(&g_MapInfo,
                        (PMapNotifyFunc)MapOnTopRowChanged,
                        (PMapNotifyFunc)MapOnLeftColumnChanged);
#endif // USE_MAP_CALLBACKS    
    
    // You can call this function to actually commit this BG to the hardware.
    // This command will throw an error if the BG that you want to init has not yet
    // been assigned a ti and mi. For more information on how to assign these to a bg,
    //see the links below.
    ham_InitBg(2, TRUE, 0, FALSE);

                   
   // Will, once activated, trigger the specified interrupt,
   // and then call the function specified
   ham_StartIntHandler(INT_TYPE_VBL,         // The Interrupts ID you want to start.
                       (void *)&vblFunc);    // The adress of a function that should be called when the interrupt is fired
   
    u8 CamScrollActive=TRUE;
    
    // Loop
    while(true)
    {
        // New Frame?
        if(g_NewFrame)
        {
            // Read current pad state
            hel_PadReadState();
            
            // Scroll the map
            if(CamScrollActive)
            {
               CamScrollActive = hel_MapScrollTo(&g_MapInfo, &g_MapCamInfo, SCROLL_SPEED, SCROLL_SPEED);
            }                             
                        
            // reset cinema scrolling structure
            if(hel_Pad.Pressed.B)
            {
               g_MapCamInfo.CurKeyPoint=0; // reset keypoint
               CamScrollActive=TRUE;
            }
            
            g_NewFrame=0;
        }
    }

    hel_MapDeInit(&g_MapInfo);
    hel_PadReset();
    
    ham_DeInitBg(0);
    
    return 0;
}

//
// This function is triggered whenever HEL's mapsystem has drawn a new row
// Function was previously setup using hel_MapSetCallbacks()
//
// Please read hel_MapSetCallbacks() docu.
//
void MapOnTopRowChanged(TMapScrollInfo *pMapInfo, u32 X, u32 Y)
{
   ham_DrawText(1, 1, "MapOnTopRowChanged    ");
   ham_DrawText(1, 2, "X: %d, Y: %d  ", X, Y);
}

//
// This function is triggered whenever HEL's mapsystem has drawn a new column
// Function was previously setup using hel_MapSetCallbacks()
//
// Please read hel_MapSetCallbacks() docu.
//
void MapOnLeftColumnChanged(TMapScrollInfo *pMapInfo, u32 X, u32 Y)
{
   ham_DrawText(1, 1, "MapOnLeftColumnChanged");
   ham_DrawText(1, 2, "X: %d, Y: %d  ", X, Y);
}

/*********************************************************************************
 * vblFunc(void)
 *
 * This function is called whenever the GBA is about
 * to draw a new picture onto the screen.
 ********************************************************************************/
void vblFunc()
{
	// Signify new frame
	g_NewFrame=1;
	g_Frames++;
}

/* END OF FILE */
